<?php
define('VIEW_PATH', realpath(__DIR__));

class ViewModel
{
    public function renderController($controller, $action)
    {
        $path = sprintf('%s/%s/%s.phtml', VIEW_PATH, $this->getFolder($controller), $this->getFolder($action));
        
        if (file_exists($path))
        {
            return $this->getFileContent($path);
        }
        else
        {
            throw new Exception('Not found view: ' . $path);
        }        
    }
    
    
    /**
     * Get Folder path
     * 
     * @param string $name
     * @return string
     */
    protected function getFolder($name)
    {
        $name = preg_replace('/(Controller$)|(Action$)/', '', $name);
        
        return preg_replace_callback('/([A-Z])/', function($m){return '-' . strtolower($m[1]);}, lcfirst($name));
    }
    
    
    protected function getFileContent($file)
    {
        ob_start();
        include($file);
        $contents = ob_get_contents(); // data is now in here
        ob_end_clean();
        
        return $contents;
    }
    
    
    /**
     * Get data
     * 
     * @param string $name
     * @return NULL|mixed
     */
    public function __get($name)
    {
        if (isset($this->$name)) {
            return $this->$name;
        }
    
        return null;
    }
    
    
    /**
     * Directly assigns a variable to the view script.
     *
     * Checks first to ensure that the caller is not attempting to set a
     * protected or private member (by checking for a prefixed underscore); if
     * not, the public member is set; otherwise, an exception is raised.
     *
     * @param string $key The variable name.
     * @param mixed $val The variable value.
     * @return void
     * @throws Exception if an attempt to set a private or protected
     * member is detected
     */
    public function __set($key, $val)
    {
        $this->setVariable($key, $val);
    }
    
    
    /**
     * Directly assigns a variable to the view script.
     *
     * Checks first to ensure that the caller is not attempting to set a
     * protected or private member (by checking for a prefixed underscore); if
     * not, the public member is set; otherwise, an exception is raised.
     *
     * @param string $key The variable name.
     * @param mixed $val The variable value.
     * @return void
     * @throws Exception if an attempt to set a private or protected
     * member is detected
     */
    public function setVariable($key, $val)
    {
        if ('_' != substr($key, 0, 1)) {
            $this->$key = $val;
            return $this;
        }
        
        throw new Exception('Setting private or protected class members is not allowed');        
    }
    
    
    /**
     * Set variables to view
     * 
     * @param array $params
     */
    public function setVariables(array $params)
    {
        foreach ($params as $key => $val) {
            $this->setVariable($key, $val);
        }
        
        return $this;
    }
    
    
    /**
     * Return list of all assigned variables
     *
     * Returns all public properties of the object. Reflection is not used
     * here as testing reflection properties for visibility is buggy.
     *
     * @return array
     */
    public function getVars()
    {
        $vars   = get_object_vars($this);
        foreach ($vars as $key => $value) {
            if ('_' == substr($key, 0, 1)) {
                unset($vars[$key]);
            }
        }
    
        return $vars;
    }
    
    
    /**
     * Clear all assigned variables
     *
     * Clears all variables assigned to Zend_View either via {@link assign()} or
     * property overloading ({@link __set()}).
     *
     * @return void
     */
    public function clearVars()
    {
        $vars   = get_object_vars($this);
        foreach ($vars as $key => $value) {
            if ('_' != substr($key, 0, 1)) {
                unset($this->$key);
            }
        }
    }
}